import { world, system, BlockPermutation, ItemStack } from "@minecraft/server";
import { verticalSlabIdentifiers } from "./vertical_slab_identifiers.js";
import { getGamemode } from "./lib/get_gamemode.js";

const cardinalDirectionOpposites = {
  north: "south",
  east: "west",
  south: "north",
  west: "east",
  up: "down",
  down: "up",
};

const showDebugMessages = false;

function debugMessage(player, message) {
  if (showDebugMessages) {
    console.log("[Enchanted Vertical Slabs]: " + message)
    try {
      player.sendMessage(message);
    } catch (e) {}
  }
}

function isPlayerInBlockPosition(player, block) {
  if (
    Math.floor(player.location.x) == Math.floor(block.location.x) &&
    (Math.floor(player.location.y) == Math.floor(block.location.y) || Math.floor(player.location.y + 1) == Math.floor(block.location.y)) &&
    Math.floor(player.location.z) == Math.floor(block.location.z)
  ) {
    return true;
  }
  return false;
}

world.beforeEvents.itemUseOn.subscribe((event) => {
  const block = event.block;
  const itemStackID = event.itemStack.typeId;

  if (verticalSlabIdentifiers.includes(event.itemStack.typeId) && block.hasTag(itemStackID)) {
    const player = event.source;
    const doubleSlabPermutation = BlockPermutation.resolve(itemStackID, { "evs:double": true });

    const faceInteractedOn = event.blockFace.toLowerCase();
    const slabRotation = block.permutation.getState("minecraft:cardinal_direction").toString().toLowerCase();

    if (faceInteractedOn === "up" || faceInteractedOn === "down") {
      debugMessage(player, `interacted on up or down face`);
      return;
    }

    // didnt interact on correct face to combine
    if (cardinalDirectionOpposites[slabRotation] !== faceInteractedOn) {
      debugMessage(player, `didnt interact on correct face to combine `);
      return;
    }

    // interacted on double slab of same type
    if (block.permutation.matches(itemStackID, { "evs:double": true })) {
      return;
    }

    // combine slab
    if (player.isSneaking === false) {
      event.cancel = true;

      system.run(() => {
      debugMessage(player, `Interacted on: '${itemStackID}'`);
        // player is inside slab
        if ( isPlayerInBlockPosition(player, block) || block.permutation.getState("evs:double")) {
          return;
        }

        event.block.setPermutation(doubleSlabPermutation);

        // remove 1 slab from hand
        let handItem = player.getComponent("equippable").getEquipment("Mainhand")?.clone();
        if (handItem?.typeId === itemStackID && getGamemode(player) !== "creative") {
          if (handItem.amount >= 2) {
            handItem.amount -= 1;
            player.getComponent("equippable").setEquipment("Mainhand", handItem);
          } else {
            player.getComponent("equippable").setEquipment("Mainhand", undefined);
          }
        }

      });
    }
  }
});

world.afterEvents.playerBreakBlock.subscribe((event) => {
  const brokenBlockPermutation = event.brokenBlockPermutation;
  const player = event.player;

  if ( getGamemode(player) == "creative" ) {
    return;
  }
  
  for (let i = 0; i < verticalSlabIdentifiers.length; i++) {
    let currentID = verticalSlabIdentifiers[i].toString();

    if( brokenBlockPermutation.matches(currentID, { "evs:double": true }) ) {
      debugMessage(player, `broken block is '${currentID}["evs:double"=true]'`);
      spawnItemStack(new ItemStack(currentID, 1), event.dimension, event.block.location);
      break;
    }
  }
})

function spawnItemStack(itemStack, dimension, location) {
  dimension.spawnItem(itemStack, location)
}